"""Rejpseplan CLI application entry point."""

import importlib.metadata


def main():
    """Main function to display the version and a stylized header for the pyRejseplan application.
    This function retrieves the current version of the pyRejseplan package using importlib.metadata,
    constructs a stylized ASCII header with the version information, and prints it to the console."""

    version = importlib.metadata.version('pyRejseplan')
    headline = r"""
####################################################################
#                                                                  #
#                  _____      _          _____  _                  #
#                 |  __ \    (_)        |  __ \| |                 #
#      _ __  _   _| |__) |___ _ ___  ___| |__) | | __ _ _ __       #
#     | '_ \| | | |  _  // _ \ / __|/ _ \  ___/| |/ _` | '_ \      #
#     | |_) | |_| | | \ \  __/ \__ \  __/ |    | | (_| | | | |     #
#     | .__/ \__, |_|  \_\___| |___/\___|_|    |_|\__,_|_| |_|     #
#     | |     __/ |         _/ |                                   #
#     |_|    |___/         |__/                                    #
"""
    border = '#' * 68
    blank = '#' + ' ' * 66 + '#' + '\n'
    version_info = '#' + f'pyRejseplan version: {version}'.center(66) + '#\n'
    header = f'{headline}{blank}{version_info}{border}'
    print(header)


if __name__ == '__main__':
    main()



import logging
import os
import sys
import argparse

# from py_rejseplan.locationhandler import LocationHandler
from py_rejseplan.departureboard import DepartureBoard

DEBUG = False

log_lvl = logging.INFO

rootlogger: logging.Logger = logging.getLogger()
rootlogger.setLevel(log_lvl)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
rootlogger.addHandler(ch)

parser = argparse.ArgumentParser(
    description='Test script for testing location functionality for Rejseplanen API'
)
parser.add_argument('-d', '--debug', action='store_true', help='Enable debug mode')

args = parser.parse_args()

if args.debug:
    rootlogger.info('DEBUG flag set')
    log_lvl = logging.DEBUG
    rootlogger.setLevel(log_lvl)
    ch.setLevel(log_lvl)
    DEBUG = True

KEY = None
with open(os.path.join(os.getcwd(), 'rejseplan.key'), encoding='utf-8') as keyfile:
    for line in keyfile.readlines():
        if line.startswith('KEY:'):
            KEY = line.strip('KEY:').strip()
if not KEY:
    print('Auth key not found')
    sys.exit(1)
rootlogger.info('Auth key found')

if args.debug:
    departure_board = DepartureBoard(KEY, r'requestData\mdbRoskildeSt.pkl')
else:
    departure_board = DepartureBoard(KEY)

departure_board._stop_ids = [8600617, 8600794]

departure_board.get_departures()



"""Conftest document containing setup of fixtures for pytest"""

import os
import pytest


@pytest.fixture(name='key')
def fixture_key():
    """Reading authkey from auth file in root

    Returns:
        auth key: str
    """
    _key = None
    try:
        with open(
            os.path.join(os.getcwd(), 'rejseplan.key'), encoding='utf-8'
        ) as keyfile:
            for line in keyfile.readlines():
                if line.startswith('KEY:'):
                    _key = line.strip('KEY:').strip()
    except FileNotFoundError as err:
        print(f'{err.filename} not found, using dummy key')
        _key = 'DUMMY_KEY'
    if not _key:
        print('Auth key not found, using dummy')
        _key = 'DUMMY_KEY'
    return _key



# -*- coding: utf-8 -*-
"""Product class for parsing XML data from the Rejseplanen API."""
from typing import Optional
from pydantic_xml import BaseXmlModel, attr, element
import py_rejseplan.dataclasses.constants as constants


class Product(
    BaseXmlModel,
    tag='Product',
    ns="",
    nsmap=constants.NSMAP
):
    """Product class for parsing XML data from the Rejseplanen API.
    This class is used to represent the product data returned by the API.
    It extends the BaseXmlModel from pydantic_xml to provide XML parsing capabilities.
    """
    name: str = attr()
    internalName: str = attr()
    displayNumber: str = attr()
    num: str = attr()
    catOut: str = attr()
    catIn: str = attr()
    catCode: str = attr()
    cls: str = attr()
    catOutS: str = attr()
    catOutL: str = attr()
    operatorCode: Optional[str] = attr(default="")
    operator: Optional[str] = attr(default="")
    admin: str = attr()
    routeIdxFrom: int = attr()
    routeIdxTo: int = attr()
    matchId: str = attr()
    icon: dict[str, str] = element(
        default_factory=dict,
        tag='icon'
    )
    operatorInfo: dict[str, str] = element(
        default_factory=dict,
        tag='operatorInfo'
    )

# <Product name="Re 54541" internalName="Re 54541" displayNumber="54541" num="54541"
#     catOut="Re" catIn="004" catCode="2" cls="4" catOutS="004" catOutL="Re"
#     operatorCode="DSB" operator="DSB" admin="000002" routeIdxFrom="19" routeIdxTo="24"
#     matchId="54541">
#     <icon res="prod_ic" />
#     <operatorInfo name="DSB" nameS="DSB" nameN="DSB" nameL="DSB" />
# </Product>


import logging
import pytest

from py_rejseplan.api.departures import departuresAPIClient

_LOGGER = logging.getLogger(__name__)


@pytest.fixture
def departures_api_client(key):
    """Fixture to create a departures API client."""
    auth = key
    departures_api_client = departuresAPIClient(auth_key=auth)
    return departures_api_client


"""API error when Rejseplanen API v2.0 returns an error."""


class RPAPIError(BaseException):
    """Raised when the API returned an error."""

    def __init__(self, message, status_code=None):
        super().__init__(message)
        self.message = message
        self.status_code = status_code

    def __str__(self):
        if self.status_code:
            return f'RPAPIError (status code: {self.status_code}): {self.message}'
        return f'RPAPIError: {self.message}'



from typing import List, Optional
from pydantic_xml import BaseXmlModel, attr, element
import py_rejseplan.dataclasses.constants as constants

class Note(BaseXmlModel, tag='Note', nsmap=constants.NSMAP):
    key: str = attr()
    type: str = attr()
    priority: Optional[int] = attr(default=None)
    routeIdxFrom: Optional[int] = attr(default=None)
    routeIdxTo: Optional[int] = attr(default=None)
    txtN: Optional[str] = attr()
    content: Optional[str] = element(tag='', default=None) # Captures the text content


class Notes(BaseXmlModel, tag='Notes', nsmap=constants.NSMAP):
    note: List[Note] = element(tag='Note', default_factory=list)


import logging
from pydantic_xml import BaseXmlModel, attr

_LOGGER = logging.getLogger(__name__)

class JourneyDetailRef(
    BaseXmlModel,
    tag='JourneyDetailRef',
    ns="",
    nsmap=None
):
    """JourneyDetailRef class for parsing XML data from the Rejseplanen API.
    This class is used to represent the journey detail reference data returned by the API.
    It extends the BaseXmlModel from pydantic_xml to provide XML parsing capabilities.
    """
    ref: str = attr()


import datetime
import logging
from typing import Optional
from pydantic_xml import BaseXmlModel, attr, element

import py_rejseplan.dataclasses.constants as constants

from .product_at_stop import ProductAtStop
from .product import Product
from .journey_detail_ref import JourneyDetailRef
from .note import Notes
from .enums import JourneyStatusType, PrognosisType, DepartureType, RealtimeDataSourceType

_LOGGER = logging.getLogger(__name__)


class Departure(
    BaseXmlModel,
    tag='Departure',
    ns="",
    nsmap=constants.NSMAP,
    search_mode='unordered',
    ):
    """Departure class for parsing XML data from the Rejseplanen API.
    This class is used to represent the departure data returned by the API.
    It extends the BaseXmlModel from pydantic_xml to provide XML parsing capabilities.
    """
    name: str = attr()
    type: DepartureType = attr(default=None, tag='type',)
    stop: str = attr()
    stopid: str = attr()
    stopExtId: int = attr()
    lon: Optional[float] = attr(default=None)
    lat: Optional[float] = attr(default=None)
    isMainMast: Optional[bool] = attr(default=False)
    hasMainMast: Optional[bool] = attr(default=False)
    mainMastId: Optional[str] = attr(default=None, tag='mainMastId')
    mainMastExtId: Optional[str] = attr(default=None, tag='mainMastExtId')
    mainMastLon: Optional[float] = attr(default=None, tag='mainMastLon')
    mainMastLat: Optional[float] = attr(default=None, tag='mainMastLat')
    mainMastAlt: Optional[int] = attr(default=None, tag='mainMastAlt')
    prognosisType: Optional[PrognosisType] = attr(default=None,tag='prognosisType')
    time: datetime.time = attr()
    scheduledTimeChanged: Optional[bool] = attr(default=False, tag='scheduledTimeChanged')
    date: datetime.date = attr()
    tz: Optional[int] = attr(default=0, tag='tz')
    track: Optional[str] = attr(default=None)
    trackHidden: bool = attr(default=False, tag='trackHidden')

    rtTime: Optional[datetime.time] = attr(tag='rtTime', default=None)
    rtDate: Optional[datetime.date] = attr(tag='rtDate', default=None)
    rtTz: Optional[int] = attr(tag='rtTz', default=0)
    rtTrack: Optional[int] = attr(tag='rtTrack', default=-1)
    rttrackHidden: bool = attr(default=False, tag='rtTrackHidden')
    cancelled: Optional[bool] = attr(default=False)
    partCancelled: Optional[bool] = attr(default=False, tag='partCancelled')
    reachable: Optional[bool] = attr(default=True)
    redirected: Optional[bool] = attr(default=False)
    direction: Optional[str] = attr(default=None)
    directionFlag: int = attr()
    directionExtId: Optional[str] = attr(default=None, tag='directionExtId')
    timeAtArrival: Optional[datetime.time] = attr(default=None, tag='timeAtArrival')
    dateAtArrival: Optional[datetime.date] = attr(default=None, tag='dateAtArrival')
    rtTimeAtArrival: Optional[datetime.time] = attr(default=None, tag='rtTimeAtArrival')
    rtDateAtArrival: Optional[datetime.date] = attr(default=None, tag='rtDateAtArrival')
    isFastest: Optional[bool] = attr(default=False, tag='isFastest')
    isBorderStop: Optional[bool] = attr(default=False, tag='isBorderStop')
    isTurningPoint: Optional[bool] = attr(default=False, tag='isTurningPoint')
    entry: Optional[bool] = attr(default=None)
    rtCnclDataSourceType: Optional[RealtimeDataSourceType] = attr(default=None, tag='rtCnclDataSourceType')
    uncertainDelay: Optional[bool] = attr(default=False, tag='uncertainDelay')

    # Subelements
    originStop: Optional['Departure'] = element(
        default=None,
        tag='OriginStop'
    )
    destinationStop: Optional['Departure'] = element(
        default=None,
        tag='DestinationStop'
    )
    journeyDetailRef: JourneyDetailRef = element(
        default_factory=JourneyDetailRef,
        tag='JourneyDetailRef'
    )
    journeyStatus: JourneyStatusType = element(
        default_factory=None,
        tag='JourneyStatus'
    )
    productAtStop: ProductAtStop = element(
        default_factory=list,
        tag='ProductAtStop'
    )
    product: Product = element(
        default_factory=list,
        tag='Product'
    )
    notes: Notes = element(
        tag='Notes',  # This navigates through the Notes container
    )

    messages: list[str] = element(
        default_factory=list,
        tag='Messages'
    )

    directions: list[str] = element(
        default_factory=list,
        tag='Directions'
    )

    platform: dict[str, str] = element(
        default_factory=dict,
        tag='platform'
    )
    rtPlatform: dict[str, str] = element(
        default_factory=dict,
        tag='rtPlatform'
    )


# <xs:element name="OriginStop" type="StopType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="DestinationStop" type="StopType" minOccurs="0" maxOccurs="1"/>
# <xs:element ref="JourneyDetailRef"/>
# <xs:element name="JourneyStatus" type="JourneyStatusType" minOccurs="0"/>
# <xs:element name="ProductAtStop" type="ProductType" minOccurs="0" maxOccurs="1">
# <xs:element name="Product" type="ProductType" minOccurs="0" maxOccurs="unbounded">
# <xs:documentation>Product information from requested departure stop to the last stop of the service. In case of product information changes along, multiple entries are possible.</xs:documentation>
# <xs:element ref="Notes" minOccurs="0"/>
# <xs:element ref="Messages" minOccurs="0"/>
# <xs:element ref="Directions" minOccurs="0"/>
# <xs:element name="altId" type="xs:string" minOccurs="0" maxOccurs="unbounded">
# <xs:element name="mainMastAltId" type="xs:string" minOccurs="0" maxOccurs="unbounded">
# <xs:element ref="Stops" minOccurs="0"/>
# <xs:element name="Occupancy" type="OccupancyType" minOccurs="0" maxOccurs="unbounded"/>
# <xs:element name="ParallelJourneyRef" type="ParallelJourneyRefType" minOccurs="0" maxOccurs="unbounded"/>
# <xs:element name="referencedJourney" type="ReferencedJourneyType" minOccurs="0" maxOccurs="unbounded">
# <xs:element name="platform" type="PlatformType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="rtPlatform" type="PlatformType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="mainMast" type="StopLocation" minOccurs="0">
# <xs:attributeGroup ref="attlist.Departure"/>

# ==== ATTLIST DEPARTURE ====

# <xs:attribute name="name" use="required">  # DONE
# <xs:documentation>Specifies the name of the departing journey (e.g. "Bus 100") as used for display. </xs:documentation>  

# <xs:attribute name="type" use="required">  # DONE
# <xs:documentation>The attribute type specifies the type of departs location. Valid values are ST (stop/station), ADR (address), POI (point of interest), CRD (coordinate), MCP (mode change point) or HL (hailing point). </xs:documentation>

# <xs:simpleType>
#     <xs:restriction base="xs:string">
#         <xs:enumeration value="ST"/>
#         <xs:enumeration value="ADR"/>
#         <xs:enumeration value="POI"/>
#         <xs:enumeration value="CRD"/>
#         <xs:enumeration value="MCP"/>
#         <xs:enumeration value="HL"/>
#     </xs:restriction>
# </xs:simpleType>

# <xs:attribute name="stop" type="xs:string" use="required">  # DONE
# <xs:documentation>Contains the name of the stop/station.</xs:documentation>

# <xs:attribute name="stopid" type="xs:string" use="required">
# <xs:documentation>Contains the ID of the stop/station.</xs:documentation>  # DONE

# <xs:attribute name="stopExtId" type="xs:string" use="optional">
# <xs:documentation>External ID of this stop/station</xs:documentation>  # DONE

# <xs:attribute name="lon" type="xs:decimal" use="optional">  # DONE
# <xs:documentation>The WGS84 longitude of the geographical position of this stop/station.</xs:documentation>

# <xs:attribute name="lat" type="xs:decimal" use="optional">  # DONE    
# <xs:documentation>The WGS84 latitude of the geographical position of this stop/station.</xs:documentation>

# <xs:attribute name="alt" type="xs:int" use="optional">  # DONE
# <xs:documentation>The altitude of the geographical position of this stop/station.</xs:documentation>

# <xs:attribute name="isMainMast" type="xs:boolean"> # DONE
# <xs:documentation>True if this stop is a main mast.</xs:documentation>

# <xs:attribute name="hasMainMast" type="xs:boolean">  # DONE
# <xs:documentation>True if this stop belongs to a main mast.</xs:documentation>

# <xs:attribute name="mainMastId" type="xs:string">  # DONE
# <xs:documentation>Deprecated. Use mainMast structure instead. ID of the main mast this stop belongs to.</xs:documentation>

# <xs:attribute name="mainMastExtId" type="xs:string" use="optional">  # DONE
# <xs:documentation>Deprecated. Use mainMast structure instead. External ID of the main mast this stop belongs to.</xs:documentation>

# <xs:attribute name="mainMastLon" type="xs:decimal" use="optional">  # DONE
# <xs:documentation>Deprecated. Use mainMast structure instead. The WGS84 longitude of the geographical position of the main mast this stop/station. </xs:documentation>

# <xs:attribute name="mainMastLat" type="xs:decimal" use="optional">  # DONE
# <xs:documentation>Deprecated. Use mainMast structure instead. The WGS84 latitude of the geographical position of the main mast this stop/station. </xs:documentation>

# <xs:attribute name="mainMastAlt" type="xs:int" use="optional">  # DONE
# <xs:documentation>Deprecated. Use mainMast structure instead. The altitude of the geographical position of the main mast this stop/station.</xs:documentation>

# <xs:attribute name="prognosisType" type="PrognosisType" use="optional">  # DONE
# <xs:documentation>Prognosis type of departure date and time.</xs:documentation>

# <xs:attribute name="time" type="xs:string" use="required">    # DONE
# <xs:documentation>Time in format hh:mm[:ss]</xs:documentation>

# <xs:attribute name="scheduledTimeChanged" type="xs:boolean" use="optional" default="false">   # DONE
# <xs:documentation>Scheduled time changed.</xs:documentation>

# <xs:attribute name="date" type="xs:string" use="required"> # DONE
# <xs:documentation>Date in format YYYY-MM-DD.</xs:documentation>

# <xs:attribute name="tz" type="xs:int" use="optional" default="0"> # DONE
# <xs:documentation>Time zone information in the format +/- minutes</xs:documentation>

# <xs:attribute name="track" type="xs:string" use="optional">
# <xs:documentation>Track information, if available.</xs:documentation>

# <xs:attribute name="trackHidden" type="xs:boolean" default="false">
# <xs:documentation>True if track information is hidden by data.</xs:documentation>

# <xs:attribute name="rtTime" type="xs:string" use="optional">
# <xs:documentation>Realtime time in format hh:mm[:ss] if available.</xs:documentation>

# <xs:attribute name="rtDate" type="xs:string" use="optional">
# <xs:documentation>Realtime date in format YYYY-MM-DD, if available.</xs:documentation>

# <xs:attribute name="rtTz" type="xs:int" use="optional" default="0">
# <xs:documentation>Realtime time zone information in the format +/- minutes, if available.</xs:documentation>

# <xs:attribute name="rtTrack" type="xs:string" use="optional">
# <xs:documentation>Realtime track information, if available.</xs:documentation>

# <xs:attribute name="rtTrackHidden" type="xs:boolean" default="false">
# <xs:documentation>True if track information is hidden by realtime data.</xs:documentation>

# <xs:attribute name="cancelled" type="xs:boolean" use="optional" default="false">
# <xs:documentation>Will be true if this journey is cancelled</xs:documentation>

# <xs:attribute name="partCancelled" type="xs:boolean" use="optional" default="false">
# <xs:documentation>Will be true if this journey is partially cancelled.</xs:documentation>

# <xs:attribute name="reachable" type="xs:boolean" use="optional" default="true">
# <xs:documentation>Will be true if this journey is reachable. A journey is considered reachable if either the follow-up journey is reachable based on the scheduled time (default without realtime) or the followup journey is not reachable regarding realtime situation but reported as reachable explicitly.</xs:documentation>

# <xs:attribute name="redirected" type="xs:boolean" use="optional" default="false">
# <xs:documentation>Will be true if this journey is redirected. A journey is considered as redirected if structural changes (e.g. additional/removed stop, change of scheduled times, ...) have been made.</xs:documentation>

# <xs:attribute name="direction" type="xs:string" use="optional">
# <xs:documentation>Direction information. This is the last stop of the journey. Get the full journey of the train or bus with the JourneyDetails service. </xs:documentation>

# <xs:attribute name="directionFlag" type="xs:string">
# <xs:documentation>Direction flag of the journey.</xs:documentation>

# <xs:attribute name="directionExtId" type="xs:string" use="optional">
# <xs:documentation>External ID of direction stop/station</xs:documentation>

# <xs:attribute name="timeAtArrival" type="xs:string" use="optional">
# <xs:documentation>Time in format hh:mm[:ss] the services arrives at the destination.</xs:documentation>

# <xs:attribute name="dateAtArrival" type="xs:string" use="optional">
# <xs:documentation>Date in format YYYY-MM-DD the services arrives at the destination.</xs:documentation>

# <xs:attribute name="rtTimeAtArrival" type="xs:string" use="optional">
# <xs:documentation>Realtime time in format hh:mm[:ss] the services arrives at the destination.</xs:documentation>

# <xs:attribute name="rtDateAtArrival" type="xs:string" use="optional">
# <xs:documentation>Realtime date in format YYYY-MM-DD the services arrives at the destination. </xs:documentation>

# <xs:attribute name="isFastest" type="xs:boolean" use="optional">
# <xs:documentation>Services is 'fastest service to' location.</xs:documentation>

# <xs:attribute name="isBorderStop" type="xs:boolean" use="optional" default="false">
# <xs:documentation>Will be true if this stop is a border stop</xs:documentation>

# <xs:attribute name="isTurningPoint" type="xs:boolean" use="optional" default="false">
# <xs:documentation>Will be true if this stop is a turning point</xs:documentation>

# <xs:attribute name="entry" type="xs:boolean" use="optional">
# <xs:documentation>True, if the stop is an entry point.</xs:documentation>

# <xs:attribute name="rtCnclDataSourceType" type="RealtimeDataSourceType" use="optional">
# <xs:documentation>Realtime data source that the stop cancellation originates from</xs:documentation>

# <xs:attribute name="uncertainDelay" type="xs:boolean" use="optional" default="false">
# <xs:documentation>The journey stopped or is waiting somewhere along its path and some journey stops contain an uncertain delay.</xs:documentation>



import logging

from py_rejseplan.api.departures import departuresAPIClient

_LOGGER = logging.getLogger(__name__)

def test_get_departures(departures_api_client: departuresAPIClient):
    """Test the request method of departuresAPIClient."""

    _LOGGER.debug('Testing request method')
    # Call the request method with a sample stop ID
    # stop_id = [8600617]
    stop_id = [8600695, 8600617]
    departures, response = departures_api_client.get_departures(stop_id)
    assert response is not None, "Response should not be None"
    assert response.status_code == 200, f"Expected status code 200, got {response.status_code}"
    assert departures is not None, "Departures should not be None"
 
def test_validate_auth_key(departures_api_client: departuresAPIClient):
    """Test the validate_auth_key method of departuresAPIClient."""

    _LOGGER.debug('Testing validate_auth_key method')
    # Call the validate_auth_key method
    is_valid = departures_api_client.validate_auth_key()
    assert is_valid, "Authorization key should be valid"

def test_validate_auth_key_invalid(departures_api_client: departuresAPIClient):
    """Test the validate_auth_key method with an invalid key.
    This test simulates an invalid authorization key and checks if the method correctly identifies it as invalid.
    should use xfail to indicate that this test is expected to fail.
    """

    _LOGGER.debug('Testing validate_auth_key with invalid key')
    # Set an invalid auth key
    departures_api_client.headers['Authorization'] = 'Bearer invalid_key'
    is_valid = departures_api_client.validate_auth_key()
    assert not is_valid, "Authorization key should be invalid"


import requests
import logging

_LOGGER = logging.getLogger(__name__)

class baseAPIClient():
    """Base class for API clients.
    This class provides a method to construct headers for API requests.
    """
    def __init__(self, base_url:str, auth_key: str, timeout:int = 10) -> None:
        """Initialize the base API client with the provided base URL, authorization key, and optional timeout.

        Args:
            base_url (str): The base URL for the API.
            auth_key (str): The authorization key to be used in headers.
            timeout (int, optional): Timeout for API requests in seconds. Defaults to 10.
        """
        _LOGGER.debug('Initializing baseAPIClient')
        self.base_url = base_url
        self.headers = {'Authorization': f'Bearer {auth_key}'}
        self.timeout = timeout

    def _get(self, service:str, params:dict) -> requests.Response:
        """Make a GET request to the specified service with the given parameters.

        Args:
            service (str): The API service endpoint.
            params (dict): The parameters to include in the request.
        """
        url = self.base_url + service
        _LOGGER.debug('Making request to %s with params: %s', url, params)
        try:
            response = requests.get(url, params=params, headers=self.headers, timeout=self.timeout)
            response.raise_for_status()  # Raise an error for bad responses
        except requests.exceptions.RequestException as ex:
            _LOGGER.error('Request failed: %s', ex)
            raise ex
        if response.status_code == requests.codes['OK']:
            _LOGGER.debug('Request successful: %s', response.status_code)
            return response
        return None
    
    def validate_auth_key(self) -> bool:
        """Validate the authorization key by making a simple request to the API.

        Returns:
            bool: True if the authorization key is valid, False otherwise.
        """
        try:
            response = self._get('datainfo', params={})
            if response is not None:
                _LOGGER.debug('Authorization key is valid')
                return True
        except requests.exceptions.HTTPError as http_err:
            if http_err.response.status_code == 401:
                _LOGGER.error('Unauthorized: Invalid authorization key')
                return False
            else:
                _LOGGER.error('HTTP error occurred: %s', http_err)
                return False
        except requests.exceptions.RequestException as ex:
            _LOGGER.error('Request failed: %s', ex)
            return False


"""Exception for issues related to connection to Rejseplanen REST API"""


class RPConnectionError(BaseException):
    """Raised in the event of a network problem."""

    def __init__(self, message, status_code=None):
        super().__init__(message)
        self.message = message
        self.status_code = status_code

    def __str__(self):
        if self.status_code:
            return (
                f'RPConnectionError (status code: {self.status_code}): {self.message}'
            )
        return f'RPConnectionError: {self.message}'



"""py_rejseplan package
This package provides various modules to interact with the Rejseplanen API,
which is used for retrieving public transportation data in Denmark.
Modules:
- constants: Contains constant values used throughout the package.
- dataclasses: Defines data structures for handling API responses.
- locationhandler: Provides functionality to handle location-based queries.
- departureboard: Manages departure board information retrieval.
- exceptions: Defines custom exceptions used in the package.
Usage:
Import the necessary modules from the package to interact with the Rejseplanen API.
Example:
    # Use locationhandler to find locations
    locations = locationhandler.find_location("Copenhagen")
"""

# from py_rejseplan import constants
# from py_rejseplan import dataclasses
# from py_rejseplan import locationhandler
# from py_rejseplan import departureboard
# from py_rejseplan import exceptions

# # logging.getLogger('pyRejseplan').addHandler(logging.NullHandler())

# __all__ = [constants, dataclasses, locationhandler, departureboard, exceptions]



import logging

import pytest

from py_rejseplan.api.departures import DepartureBoard

_LOGGER = logging.getLogger(__name__)

@pytest.fixture()
def roskilde_st_data() -> str:
    """Fixture for the Roskilde station data."""
    _LOGGER.debug("Loading Roskilde station data")
    # with open('./requestData/mdbRoskildeSt.xml', 'r', encoding='utf-8') as f:
    with open('./requestData/mdbRoskildeSt.xml', 'r', encoding='utf-8') as f:
        xml_content = f.read()
    
    return xml_content

@pytest.fixture
def departure_board(roskilde_st_data) -> DepartureBoard:
    """Fixture that parses the XML data once for all tests."""
    _LOGGER.debug('Creating DepartureBoard fixture from XML data')
    return DepartureBoard.from_xml(roskilde_st_data)


"""Constants for pyRejseplan"""

RESOURCE: str = 'https://www.rejseplanen.dk/api/'
COORDINATE_MULTIPLIER: int = 1000000
DATE_FORMAT = '%d.%m.%y'
TIME_FORMAT = '%H:%M'
NS = {'ns0': 'http://hacon.de/hafas/proxy/hafas-proxy'}



import logging
from typing import Optional
from pydantic_xml import BaseXmlModel, attr, element
import py_rejseplan.dataclasses.constants as constants

class ProductAtStop(
    BaseXmlModel,
    tag='ProductAtStop',
    ns="",
    nsmap=constants.NSMAP,
    search_mode='unordered',
):
    """ProductAtStop class for parsing XML data from the Rejseplanen API.
    This class is used to represent the product at stop data returned by the API.
    It extends the BaseXmlModel from pydantic_xml to provide XML parsing capabilities.
    """
    # attributes
    name: Optional[str] = attr(default=str)
    internalName: Optional[str] = attr(default=str)
    addName: Optional[str] = attr(default="", tag='addName')
    displayNumber: Optional[str] = attr(default="", tag='displayNumber')
    num: Optional[int] = attr(default=str)
    line: Optional[str] = attr(default=str)
    lineId: Optional[str] = attr(default=str)
    lineHidden: bool = attr(default=False, tag='lineHidden')
    catOut: Optional[str] = attr(default=str)
    catIn: Optional[str] = attr(default=str)
    catCode: Optional[str] = attr(default=str)
    cls: Optional[str] = attr(default=str)
    catOutS: Optional[str] = attr(default=str)
    catOutL: Optional[str] = attr(default=str)
    operatorCode: Optional[str] = attr(default=str)
    operator: Optional[str] = attr(default=str)
    admin: Optional[str] = attr(default=str)
    routeIdxFrom: int = attr(default=-1, tag='routeIdxFrom')
    routeIdxTo: int = attr(default=-1, tag='routeIdxTo')
    matchId: Optional[str] = attr(default=str)

    #Customer specific attributes
    tarGr: Optional[str] = attr(default=str, tag='tarGr')
    surcharge: Optional[str] = attr(default=str, tag='surcharge')
    outCtrl: Optional[str] = attr(default=str, tag='outCtrl')
    locTraffic: Optional[str] = attr(default=str, tag='locTraffic')
    shipTraffic: Optional[str] = attr(default=str, tag='shipTraffic')

    icon: dict[str, str] = element(
        default_factory=dict,
        tag='icon'
    )

    status: dict[str, str] = element(
        default_factory=dict,
        tag='status'
    )

    fromLocation: dict[str, str] = element(
        default_factory=dict,
        tag='fromLocation'
    )

    toLocation: dict[str, str] = element(
        default_factory=dict,
        tag='toLocation'
    )

    operatorInfo: dict[str, str] = element(
        default_factory=dict,
        tag='operatorInfo'
    )

    Note: list[dict[str, str]] = element(
        default_factory=list,
        tag='Note'
    )

    Message: list[dict[str, str]] = element(
        default_factory=list,
        tag='Message'
    )
    LineInfo: dict[str, str] = element(
        default_factory=dict,
        tag='LineInfo'
    )



# <xs:element name="icon" type="IconType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="status" type="ProductStatusType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="fromLocation" type="StopType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="toLocation" type="StopType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="operatorInfo" type="OperatorType" minOccurs="0" maxOccurs="1"/>
# <xs:element name="Note" type="Note" minOccurs="0" maxOccurs="unbounded"/>
# <xs:element name="Message" type="Message" minOccurs="0" maxOccurs="unbounded"/>
# <xs:element name="LineInfo" type="LineType" minOccurs="0" maxOccurs="1"/>

# - name: str, optional
# - internalName: str, optional
# - addName: str, optional
# - displayNumber: str, optional
# - num: str, optional
# - line: str, optional
# - lineId: str, optional
# - lineHidden: bool, default="false"
# - catOut: str, optional
# - catIn: str, optional
# - catCode: str, optional
# - cls: str, optional
# - catOutS: str, optional
# - catOutL: str, optional
# - operatorCode: str, optional
# - operator: str, optional
# - admin: str, optional
# - routeIdxFrom: int, default="-1"
# - routeIdxTo: int, default="-1"
# - matchId: str, optional

# <xs:attribute name="tarGr" type="xs:string" use="optional"/>
# <xs:attribute name="surcharge" type="xs:string" use="optional"/>
# <xs:attribute name="outCtrl" type="xs:string" use="optional"/>
# <xs:attribute name="locTraffic" type="xs:string" use="optional"/>
# <xs:attribute name="shipTraffic" type="xs:string" use="optional"/>





from pydantic_xml import BaseXmlModel, attr
import py_rejseplan.dataclasses.constants as constants
import logging

_LOGGER = logging.getLogger(__name__)

class TechnicalMessage(
    BaseXmlModel,
    tag='TechnicalMessage',
    ns="",
    nsmap=constants.NSMAP
):
    """Technical message class for parsing XML data from the Rejseplanen API.
    This class is used to represent the technical message data returned by the API.
    It extends the BaseXmlModel from pydantic_xml to provide XML parsing capabilities.
    """
    key: str = attr()
    text: str


import logging

from pydantic import ValidationError
import requests
from .base import baseAPIClient

from py_rejseplan.constants import RESOURCE as BASE_URL
from py_rejseplan.dataclasses.departure_board import DepartureBoard

_LOGGER = logging.getLogger(__name__)

class departuresAPIClient(baseAPIClient):
    """Client for the departures API.
    This class extends the baseAPIClient to provide specific functionality for the departures API.
    """
    def __init__(self, auth_key: str, timeout: int = 10) -> None:
        """Initialize the departures API client with the provided authorization key and optional timeout.

        Args:
            auth_key (str): The authorization key to be used in headers.
            timeout (int, optional): Timeout for API requests in seconds. Defaults to 10.
        """
        _LOGGER.debug('Initializing departuresAPIClient')
        super().__init__(BASE_URL, auth_key, timeout)

    def parse_response(self, response: bytes | str) -> DepartureBoard:
        """
        Parse the XML response from the API and return a dictionary representation of the data.
        Args:
            response (str): The XML response from the API.

        Returns:
            dict: Parsed data as a dictionary.
        """
        if response is None:
            _LOGGER.error('Response is None')
            return None
        try:
            departure_board = DepartureBoard.from_xml(response)
            return departure_board
        except ValidationError as ve:
            _LOGGER.error('Validation error parsing response: %s', ve)
            return None
        except Exception as e:
            _LOGGER.error('Error parsing response: %s', e)
            return None

    def get_departures(
            self,
            stop_ids: list[int],
            max_results: int = -1,
            use_bus: bool = True,
            use_train: bool = True,
            use_metro: bool = True,
        ) -> tuple[DepartureBoard, requests.Response]:
        """Get departures for the given stop IDs.
        Args:
            stop_ids (list[int]): List of stop IDs to get departures for.
            max_results (int, optional): Maximum number of results to return. Defaults to -1.
            use_bus (bool, optional): Whether to include bus departures. Defaults to True.
            use_train (bool, optional): Whether to include train departures. Defaults to True.
            use_metro (bool, optional): Whether to include metro departures. Defaults to True.

        Returns:
            tuple: (DepartureBoard, response object)
        """
        _LOGGER.debug('Getting departures for stop IDs: %s', stop_ids)
        if len(stop_ids) < 1:
            raise ValueError('Stop IDs must be provided')
        prep_id_list = "|".join(map(str, stop_ids))
        params = {
            'idList': prep_id_list,
            'maxResults': max_results,
            'useBus': use_bus,
            'useTrain': use_train,
            'useMetro': use_metro,
        }
        _LOGGER.debug('Requesting departures with params: %s', params)
        response = self._get( 'multiDepartureBoard', params=params)
        if response is None:
            _LOGGER.error('Failed to get departures')
            return None, response
        try:
            departure_board = self.parse_response(response.content)
        except Exception as e:
            _LOGGER.error('Error parsing response: %s', e)
            return None, response
        
        if departure_board is None:
            _LOGGER.error('Failed to parse departures')
            return None, response
        
        return departure_board, response


# -*- coding: utf-8 -*-
"""Test the DepartureBoard class."""
import logging

import datetime

import pytest
from py_rejseplan.dataclasses.departure import Departure
from py_rejseplan.dataclasses.departure_board import DepartureBoard


_LOGGER = logging.getLogger(__name__)

def test_basic_attributes(departure_board: DepartureBoard):
    """Test the basic attributes of the DepartureBoard."""
    assert departure_board.serverVersion == '2.49.1'
    assert departure_board.dialectVersion == "2.45-Rejseplanen"
    assert departure_board.requestId == "e22kxq84ic8ypg8x"

def test_timestamp_parsing(departure_board: DepartureBoard):
    """Test that timestamp is correctly parsed."""
    assert isinstance(departure_board.planRtTs, datetime.datetime)
    assert departure_board.planRtTs == datetime.datetime.fromisoformat("2025-05-23T11:12:39+02:00")

def test_technical_messages(departure_board: DepartureBoard):
    """Test the technical messages in the DepartureBoard."""
    assert hasattr(departure_board, 'technicalMessages')
    tech_messages = departure_board.technicalMessages.technicalMessages
    assert len(tech_messages) > 0
    
    # Test specific message if it exists
    request_time_msg = next((msg for msg in tech_messages if msg.key == "requestTime"), None)
    if request_time_msg:
        assert "2025-05-23" in request_time_msg.text, "Expected date in requestTime message"

def test_departures_list(departure_board: DepartureBoard):
    """Test that departures are correctly parsed."""
    assert hasattr(departure_board, 'departures')
    assert len(departure_board.departures) > 0

def test_first_departure_details(departure_board: DepartureBoard):
    """Test details of the first departure."""
    # Skip if no departures
    if not departure_board.departures:
        pytest.skip("No departures in test data")
        
    first_departure: Departure = departure_board.departures[0]
    assert first_departure.name is not None
    assert first_departure.stop == "Roskilde St."
    assert isinstance(first_departure.time, datetime.time)
    assert isinstance(first_departure.date, datetime.date)


import logging
from typing import Optional, List
from pydantic_xml import BaseXmlModel, attr, element
import py_rejseplan.dataclasses.constants as constants

class Stop(BaseXmlModel, tag='Stop', nsmap=constants.NSMAP, search_mode='unordered'):
    # Elements
    Notes: Optional[str] = element(default=None)
    altId: Optional[List[str]] = element(default=None)
    mainMastAltId: Optional[List[str]] = element(default=None)
    Occupancy: Optional[List[str]] = element(default=None)
    arrPlatform: Optional[str] = element(default=None)
    rtArrPlatform: Optional[str] = element(default=None)
    depPlatform: Optional[str] = element(default=None)
    rtDepPlatform: Optional[str] = element(default=None)
    weatherInformation: Optional[List[str]] = element(default=None)
    mainMast: Optional[str] = element(default=None)

    # Attributes
    name: str = attr()
    description: Optional[str] = attr(default=None)
    id: str = attr()
    extId: str = attr()
    routeIdx: Optional[int] = attr(default=None)
    lon: Optional[float] = attr(default=None)
    lat: Optional[float] = attr(default=None)
    alt: Optional[int] = attr(default=None)
    arrPrognosisType: Optional[str] = attr(default=None)
    depPrognosisType: Optional[str] = attr(default=None)
    depTime: Optional[str] = attr(default=None)
    depDate: Optional[str] = attr(default=None)
    depTz: Optional[int] = attr(default=0)
    scheduledDepTimeChanged: Optional[bool] = attr(default=False)
    arrTime: Optional[str] = attr(default=None)
    arrDate: Optional[str] = attr(default=None)
    arrTz: Optional[int] = attr(default=0)
    scheduledArrTimeChanged: Optional[bool] = attr(default=False)
    passingTime: Optional[str] = attr(default=None)
    passingDate: Optional[str] = attr(default=None)
    passingTz: Optional[int] = attr(default=0)
    arrTrack: Optional[str] = attr(default=None)
    arrTrackHidden: Optional[bool] = attr(default=False)
    depTrack: Optional[str] = attr(default=None)
    depTrackHidden: Optional[bool] = attr(default=False)
    rtDepTime: Optional[str] = attr(default=None)
    rtDepDate: Optional[str] = attr(default=None)
    rtDepTz: Optional[int] = attr(default=0)
    rtArrTime: Optional[str] = attr(default=None)
    rtArrDate: Optional[str] = attr(default=None)
    rtArrTz: Optional[int] = attr(default=0)
    rtArrTrack: Optional[str] = attr(default=None)
    rtArrTrackHidden: Optional[bool] = attr(default=False)
    rtDepTrack: Optional[str] = attr(default=None)
    rtDepTrackHidden: Optional[bool] = attr(default=False)
    rtPassingTime: Optional[str] = attr(default=None)
    rtPassingDate: Optional[str] = attr(default=None)
    rtPassingTz: Optional[int] = attr(default=0)
    cancelled: Optional[bool] = attr(default=False)
    cancelledDeparture: Optional[bool] = attr(default=False)
    cancelledArrival: Optional[bool] = attr(default=False)
    isMainMast: Optional[bool] = attr(default=None)
    hasMainMast: Optional[bool] = attr(default=None)
    mainMastId: Optional[str] = attr(default=None)
    mainMastExtId: Optional[str] = attr(default=None)
    mainMastLon: Optional[float] = attr(default=None)
    mainMastLat: Optional[float] = attr(default=None)
    mainMastAlt: Optional[int] = attr(default=None)
    alighting: Optional[bool] = attr(default=True)
    boarding: Optional[bool] = attr(default=True)
    rtAlighting: Optional[bool] = attr(default=None)
    rtBoarding: Optional[bool] = attr(default=None)
    additional: Optional[bool] = attr(default=False)
    isBorderStop: Optional[bool] = attr(default=False)
    isTurningPoint: Optional[bool] = attr(default=False)
    arrHide: Optional[bool] = attr(default=False)
    depHide: Optional[bool] = attr(default=False)
    depDir: Optional[str] = attr(default=None)
    entry: Optional[bool] = attr(default=None)
    rtCnclDataSourceType: Optional[str] = attr(default=None)
    arrUncertainDelay: Optional[bool] = attr(default=False)
    depUncertainDelay: Optional[bool] = attr(default=False)
    minimumChangeDuration: Optional[str] = attr(default=None)



from datetime import datetime
from pydantic_xml import BaseXmlModel, attr, element
import logging

import py_rejseplan.dataclasses.constants as constants

from .departure import Departure
from .technical_messages import TechnicalMessages

_LOGGER = logging.getLogger(__name__)

class DepartureBoard(
    BaseXmlModel,
    tag='DepartureBoard',
    # ns="",
    nsmap=constants.NSMAP
):
    """Departure board class for parsing XML data from the Rejseplanen API.
    This class is used to represent the departure board data returned by the API.
    It extends the BaseXmlModel from pydantic_xml to provide XML parsing capabilities.
    """
    serverVersion: str = attr()
    dialectVersion: str = attr()
    planRtTs: datetime = attr()
    requestId: str = attr()
    technicalMessages: TechnicalMessages = element(
        default_factory=list,
        tag='TechnicalMessages'
    )
    departures: list[Departure] = element(
        default_factory=list,
        tag='Departure'
    )


from enum import Enum

class PrognosisType(str, Enum):
    """Types of prognosis for departure times."""
    PROGNOSED = "PROGNOSED"
    MANUAL = "MANUAL"
    REPORTED = "REPORTED"
    CORRECTED = "CORRECTED"
    CALCULATED = "CALCULATED"

class DepartureType(str, Enum):
    """The attribute type specifies the type of departs location."""
    ST = "ST"
    ADR = "ADR"
    POI = "POI"
    CRD = "CRD"
    MCP = "MCP"
    HL = "HL"

class RealtimeDataSourceType(str, Enum):
    """Realtime data source types."""
    DEFAULT = "DEFAULT"           # Default source (undefined)
    VDV = "VDV"
    HIM = "HIM"
    HRC = "HRC"
    SIRI = "SIRI"
    UIC = "UIC"
    HRX = "HRX"
    GTFS = "GTFS"
    FIS = "FIS"
    DDS = "DDS"                  # Datendrehscheiben
    PAISA = "PAISA"              # PA-ISA
    FE = "FE"                    # FahrtenEditor
    BLACKLIST = "BLACKLIST"      # List of blacklisted trains
    ARAMIS = "ARAMIS"            # ARAMIS data source
    RTABO2 = "RTABO2"            # RTABO2 data source

class JourneyStatusType(str, Enum):
    """Contains the status of the journey."""
    P = "P"  # Planned: A planned journey. This is also the default value.
    R = "R"  # Replacement: The journey was added as a replacement for a planned journey.
    A = "A"  # Additional: The journey is an additional journey to the planned journeys.
    S = "S"  # Special: This is a special journey. The exact definition which journeys are considered special is up to the customer.
